@@ -1,4 +1,5 @@
 # Copyright (C) 2011--2013  Kipp Cannon, Chad Hanna, Drew Keppel
+# Copyright (C) 2013  Jacob Peoples
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -25,9 +26,18 @@
 
 
 import copy
+try:
+	from fpconst import NaN, NegInf, PosInf
+except ImportError:
+	# fpconst is not part of the standard library and might not be
+	# available
+	NaN = float("nan")
+	NegInf = float("-inf")
+	PosInf = float("+inf")
 import itertools
 import math
 import numpy
+import random
 from scipy import interpolate
 from scipy import optimize
 from scipy import stats
@@ -52,6 +62,8 @@ from glue import segments
 from glue.segmentsUtils import vote
 from pylal import ligolw_burca_tailor
 from pylal import ligolw_burca2
+from pylal import inject
+from pylal import progress
 from pylal import rate
 
 
@@ -99,10 +111,11 @@ def fap_after_trials(p, m):
 	#
 	# for sufficiently large n the denominator dominates and the terms
 	# in the series become small and the sum eventually converges to a
-	# stable value.  however, if m*p >> 1 it can take many terms before
-	# the series sum stabilizes, terms in the series initially grow
-	# large and an accurate result can only be obtained through careful
-	# cancellation of the large values.
+	# stable value (and if m is an integer the sum is exact in a finite
+	# number of terms).  however, if m*p >> 1 it can take many terms
+	# before the series sum stabilizes, terms in the series initially
+	# grow large and alternate in sign and an accurate result can only
+	# be obtained through careful cancellation of the large values.
 	#
 	# for large m*p we take a different approach to evaluating the
 	# result.  in this regime the result will be close to 1 so (1 -
@@ -119,8 +132,8 @@ def fap_after_trials(p, m):
 	# if p is close to 1, ln(1 - p) suffers a domain error
 	#
 
-	assert m >= 0	# m cannot be negative
-	assert 0 <= p <= 1	# p must be a valid probability
+	assert m >= 0, "m = %g cannot be negative" % m
+	assert 0 <= p <= 1, "p = %g must be between 0 and 1 inclusively" % p
 
 	if m * p < 1.0:
 		#
@@ -172,9 +185,10 @@ def trials_from_faps(p0, p1):
 	"""
 	Given the probabiity, p0, of an event occuring, and the
 	probability, p1, of at least one such event being observed after
-	some number of independent trials, solve the number of trials, m,
-	that relates the two probabilities.  The three quantities are
-	related by p1 = 1 - (1 - p0)^m.
+	some number of independent trials, solve for and return the number
+	of trials, m, that relates the two probabilities.  The three
+	quantities are related by p1 = 1 - (1 - p0)^m.  Generally the
+	return value is not an integer.
 
 	See also fap_after_trials().  Note that if p0 is 0 or 1 then p1
 	must be 0 or 1 respectively, and in both cases m is undefined.
@@ -188,7 +202,7 @@ def trials_from_faps(p0, p1):
 		# but we still can't solve for m
 		raise ValueError("m undefined")
 	if p1 == 1:
-		return float("inf")
+		return PosInf
 
 	#
 	# find range of m that contains solution.  the false alarm
@@ -346,6 +360,20 @@ lsctables.TableByName[lsctables.table.StripTableName(TrialsTable.TrialsTableTabl
 #
 
 
+#
+# Inspiral-specific CoincParamsDistributions sub-class
+#
+
+
+class ThincaCoincParamsDistributions(ligolw_burca_tailor.CoincParamsDistributions):
+	# FIXME:  this is the name used for the burca distributions.
+	# existing XML files were written using this name, so for
+	# compatibility we stick to it for now, but in the future we should
+	# pick a unique name so the files can't be confused for burca
+	# files.
+	ligo_lw_name_suffix = u"pylal_ligolw_burca_tailor_coincparamsdistributions"
+
+
 #
 # Paramter Distributions
 #
@@ -353,7 +381,7 @@ lsctables.TableByName[lsctables.table.StripTableName(TrialsTable.TrialsTableTabl
 
 class DistributionsStats(object):
 	"""
-	A class used to populate a CoincParamsDistribution instance using
+	A class used to populate a ThincaCoincParamsDistribution instance using
 	event parameter data.
 	"""
 
@@ -371,16 +399,16 @@ class DistributionsStats(object):
 	# between 4 and 26 each bin is .11 wide in SNR. So a width of 13 bins
 	# corresponds to 1.43 which is close to 1.41
 	filters = {
-		"H1_snr_chi": rate.gaussian_window2d(7, 7, sigma = 10),
-		"H2_snr_chi": rate.gaussian_window2d(7, 7, sigma = 10),
-		"H1H2_snr_chi": rate.gaussian_window2d(7, 7, sigma = 10),
-		"L1_snr_chi": rate.gaussian_window2d(7, 7, sigma = 10),
-		"V1_snr_chi": rate.gaussian_window2d(7, 7, sigma = 10)
+		"H1_snr_chi": rate.gaussian_window(7, 7, sigma = 10),
+		"H2_snr_chi": rate.gaussian_window(7, 7, sigma = 10),
+		"H1H2_snr_chi": rate.gaussian_window(7, 7, sigma = 10),
+		"L1_snr_chi": rate.gaussian_window(7, 7, sigma = 10),
+		"V1_snr_chi": rate.gaussian_window(7, 7, sigma = 10)
 	}
 
 	def __init__(self):
-		self.raw_distributions = ligolw_burca_tailor.CoincParamsDistributions(**self.binnings)
-		self.smoothed_distributions = ligolw_burca_tailor.CoincParamsDistributions(**self.binnings)
+		self.raw_distributions = ThincaCoincParamsDistributions(**self.binnings)
+		self.smoothed_distributions = ThincaCoincParamsDistributions(**self.binnings)
 		self.likelihood_pdfs = {}
 		self.target_length = 1000
 
@@ -394,16 +422,18 @@ class DistributionsStats(object):
 	@staticmethod
 	def likelihood_params_func(events, offsetvector):
 		instruments = set(event.ifo for event in events)
+		# don't allow both H1 and H2 to participate in the same
+		# coinc.  if both have participated favour H1
 		if "H1" in instruments:
 			instruments.discard("H2")
-		return dict(("%s_snr_chi" % event.ifo, (event.snr, event.chisq / event.snr**2)) for event in events if event.ifo in instruments)
+		return dict(("%s_snr_chi" % event.ifo, (numpy.clip(event.snr, 0., 99.9), event.chisq / event.snr**2)) for event in events if event.ifo in instruments)
 
 	def add_single(self, event):
 		self.raw_distributions.add_background(self.likelihood_params_func((event,), None))
 
 	def add_background_prior(self, n = 1., transition = 10., instruments = None, prefactors_range = (1.0, 10.0), df = 40, verbose = False):
 		# Make a new empty coinc param distributions
-		newdist = ligolw_burca_tailor.CoincParamsDistributions(**self.binnings)
+		newdist = ThincaCoincParamsDistributions(**self.binnings)
 		for param, binarr in newdist.background_rates.items():
 			instrument = param.split("_")[0]
 			# save some computation if we only requested certain instruments
@@ -433,7 +463,7 @@ class DistributionsStats(object):
 		# attribute, but that will slow this down
 		pfs = numpy.linspace(prefactors_range[0], prefactors_range[1], 10)
 		# Make a new empty coinc param distributions
-		newdist = ligolw_burca_tailor.CoincParamsDistributions(**self.binnings)
+		newdist = ThincaCoincParamsDistributions(**self.binnings)
 		for param, binarr in newdist.background_rates.items():
 			instrument = param.split("_")[0]
 			# save some computation if we only requested certain instruments
@@ -471,7 +501,7 @@ class DistributionsStats(object):
 		# attribute, but that will slow this down
 		pfs = numpy.linspace(prefactors_range[0], prefactors_range[1], 10)
 		# Make a new empty coinc param distributions
-		newdist = ligolw_burca_tailor.CoincParamsDistributions(**self.binnings)
+		newdist = ThincaCoincParamsDistributions(**self.binnings)
 		for param, binarr in newdist.injection_rates.items():
 			instrument = param.split("_")[0]
 			# save some computation if we only requested certain instruments
@@ -515,7 +545,17 @@ class DistributionsStats(object):
 		for name, binnedarray in itertools.chain(self.smoothed_distributions.background_rates.items(), self.smoothed_distributions.injection_rates.items()):
 			if verbose:
 				print >>sys.stderr, "%s," % name,
-			rate.filter_array(binnedarray.array, self.filters[name])
+			result = numpy.zeros_like(binnedarray.array)
+			while binnedarray.array.any():
+				workspace = numpy.copy(binnedarray.array)
+				cutoff = abs(workspace[abs(workspace) > 0]).min() * 1e4
+				binnedarray.array[abs(binnedarray.array) <= cutoff] = 0.
+				workspace[abs(workspace) > cutoff] = 0.
+				rate.filter_array(workspace, self.filters[name])
+				workspace[abs(workspace) < abs(workspace).max() * 1e-14] = 0.
+				result += workspace
+			binnedarray.array = result
+			numpy.clip(binnedarray.array, 0.0, PosInf, binnedarray.array)
 			binnedarray.array /= binnedarray.array.sum()
 		if verbose:
 			print >>sys.stderr, "done"
@@ -559,25 +599,134 @@ class DistributionsStats(object):
 	@classmethod
 	def from_xml(cls, xml, name):
 		self = cls()
-		self.raw_distributions, process_id = ligolw_burca_tailor.CoincParamsDistributions.from_xml(xml, name)
+		self.raw_distributions, process_id = ThincaCoincParamsDistributions.from_xml(xml, name)
 		# FIXME:  produce error if binnings don't match this class's binnings attribute?
 		binnings = dict((param, self.raw_distributions.zero_lag_rates[param].bins) for param in self.raw_distributions.zero_lag_rates)
-		self.smoothed_distributions = ligolw_burca_tailor.CoincParamsDistributions(**binnings)
+		self.smoothed_distributions = ThincaCoincParamsDistributions(**binnings)
 		return self, process_id
 
 	@classmethod
 	def from_filenames(cls, filenames, contenthandler = DefaultContentHandler, verbose = False):
 		self = cls()
-		self.raw_distributions, seglists = ligolw_burca_tailor.load_likelihood_data(filenames, u"gstlal_inspiral_likelihood", contenthandler = contenthandler, verbose = verbose)
+		self.raw_distributions, seglists = ligolw_burca_tailor.load_likelihood_data(filenames, ThincaCoincParamsDistributions, u"gstlal_inspiral_likelihood", contenthandler = contenthandler, verbose = verbose)
 		# FIXME:  produce error if binnings don't match this class's binnings attribute?
 		binnings = dict((param, self.raw_distributions.zero_lag_rates[param].bins) for param in self.raw_distributions.zero_lag_rates)
-		self.smoothed_distributions = ligolw_burca_tailor.CoincParamsDistributions(**binnings)
+		self.smoothed_distributions = ThincaCoincParamsDistributions(**binnings)
 		return self, seglists
 
 	def to_xml(self, process, name):
 		return self.raw_distributions.to_xml(process, name)
 
 
+#
+# Joint probability density for measured SNRs
+#
+
+
+def joint_pdf_of_snrs(inst_horiz_mapping, snr_threshold, snr_max, n_samples = 10000, bins_per_decade = 50.0, verbose = False):
+	"""
+	A function which returns a BinnedArray representing the joint
+	probability density of measuring a set of SNRs from a network of
+	instruments.  The inst_horiz_mapping is a dictionary mapping
+	instrument name (e.g., "H1") to horizon distance (arbitrary units).
+	snr_threshold is the lowest accepted SNR (must be > 3), and
+	n_samples is the number of lines over which to calculate the
+	density in the SNR space.  The axes of the PDF correspond to the
+	instruments in alphabetical order.
+	"""
+	snr_threshold = float(snr_threshold)	# just to be sure
+	snr_max = float(snr_max)	# just to be sure
+
+	# An effective threshold used in the calculations in order to simulate
+	# noise effects
+	snr_min = snr_threshold - 3.0
+	assert snr_max > snr_threshold
+	assert snr_min > 0.0
+
+	# get instrument names in alphabetical order
+	names = sorted(inst_horiz_mapping)
+	# get horizon distances and responses in that same order
+	DH = numpy.array([inst_horiz_mapping[inst] for inst in names])
+	resps = [inject.cached_detector[inject.prefix_to_name[inst]].response for inst in names]
+
+	pdf = rate.BinnedArray(rate.NDBins([rate.LogarithmicBins(snr_min, snr_max, int(round(bins_per_decade * math.log10(snr_max / snr_min))))] * len(names)))
+
+	psi = gmst = 0.0
+
+	if verbose:
+		progressbar = progress.ProgressBar("%s SNR joint PDF" % ", ".join(names))
+	else:
+		progressbar = None
+
+	for i in xrange(n_samples):
+		theta = math.acos(random.uniform(-1., 1.))
+		phi = random.uniform(0., 2. * math.pi)
+		cosi2 = random.uniform(-1., 1.)**2.
+
+		fpfc2 = numpy.array([inject.XLALComputeDetAMResponse(resp, phi, math.pi / 2. - theta, psi, gmst) for resp in resps])**2.
+
+		snr_times_D = 8. * DH * numpy.dot(fpfc2, numpy.array([(1.0 + cosi2)**2. / 4., cosi2]))**0.5
+
+		# index of instrument whose SNR grows fastest with decreasing D
+		axis = snr_times_D.argmax()
+
+		# furthest an event can be and still be above snr_min in
+		# all instruments, and the SNR that corresponds to in the
+		# instrument whose SNR grows fastest
+		snr_start = snr_times_D[axis] / (snr_times_D.min() / snr_min)
+
+		# 3 steps per bin
+		step_factor = 10**(1. / bins_per_decade / 3) - 1.
+		for snr in 10**numpy.arange(math.log10(snr_start), math.log10(snr_max), 1. / bins_per_decade / 3):
+			# "snr" is SNR in fastest growing instrument, from
+			# this the distance to the source is:
+			#
+			#	D = snr_times_D[axis] / snr
+			#
+			# and the SNRs in all instruments are:
+			#
+			#	snr_times_D / D
+			#
+			# but round-off protection is required to ensure
+			# all SNRs are within the allowed range
+			#
+			# SNR step size:
+			#	d(snr) = (10**(.1 / bins_per_decade) - 1.) * snr
+			#
+			# rate of change of D with SNR:
+			#	dD/d(snr) = -snr_times_D / snr^2
+			#	          = -D / snr
+			#
+			# relationship b/w dD and d(snr):
+			#	dD = -D / snr d(snr)
+			#	   = -D * (10**(.1 / bins_per_decade) - 1.)
+			#
+			# number of sources:
+			#	\propto D^2 |dD|
+			#	\propto D^3 * (10**(.1 / bins_per_decade) - 1.)
+			D = snr_times_D[axis] / snr
+			pdf[tuple((snr_times_D / D).clip(snr_min, PosInf))] += D**3. * step_factor
+
+		if progressbar is not None:
+			progressbar.update((i + 1.) / n_samples)
+
+	# number of bins per unit in SNR in the binnings.  For use as the
+	# width parameter in the filtering.
+	bins_per_snr_at_8 = 1. / ((10**(1. / bins_per_decade) - 1.) * 8.)
+	rate.filter_array(pdf.array,rate.gaussian_window(*([math.sqrt(2) * bins_per_snr_at_8] * len(inst_horiz_mapping))))
+	numpy.clip(pdf.array, 0, PosInf, pdf.array)
+	# set the region where any SNR is lower than the input threshold to
+	# zero before normalizing the pdf and returning.
+	range_all = slice(None,None)
+	range_low = slice(snr_min, snr_threshold)
+	for i in xrange(len(inst_horiz_mapping)):
+		slices = [range_all] * len(inst_horiz_mapping)
+		slices[i] = range_low
+		pdf[tuple(slices)] = 0
+	pdf.to_pdf()
+	return pdf
+
+
 #
 # =============================================================================
 #
@@ -610,7 +759,6 @@ def likelihood_bin_boundaries(likelihoods, probabilities, minint = 1e-2, maxint
 	return minlikelihood, maxlikelihood
 
 
-
 def possible_ranks_array(likelihood_pdfs, ifo_set, targetlen):
 	# start with an identity array to seed the outerproduct chain
 	ranks = numpy.array([1.0])
@@ -861,13 +1009,10 @@ class RankingData(object):
 			ccdf /= ccdf[0]
 			# try making ccdf + cdf == 1.  nothing really cares
 			# if this identity doesn't exactly hold, but we
-			# might as well avoid weirdness where we can.  the
-			# one whose tail is biggest gets adjusted to avoid
-			# getting negative probabilities.
-			if cdf[0] <= ccdf[-1]:
-				ccdf -= cdf[0]
-			else:
-				cdf -= ccdf[-1]
+			# might as well avoid weirdness where we can.
+			s = ccdf + cdf
+			cdf /= s
+			ccdf /= s
 			# build interpolators
 			self.cdf_interpolator[key] = interpolate.interp1d(ranks, cdf)
 			self.ccdf_interpolator[key] = interpolate.interp1d(ranks, ccdf)
@@ -901,7 +1046,7 @@ class RankingData(object):
 			rank = self.minrank[ifos]
 		tdp = float(self.cdf_interpolator[ifos](rank))
 		if tdp == 0.:
-			return float('inf')
+			return PosInf
 		try:
 			trials = max(int(self.trials_table[ifos].count), 1)
 		except KeyError:
@@ -916,7 +1061,7 @@ class RankingData(object):
 			# we don't have a livetime segment yet.  this can
 			# happen during the early start-up of an online
 			# low-latency analysis
-			return float('nan')
+			return NaN
 		return trials * -math.log(tdp) / livetime
 
 
@@ -940,9 +1085,9 @@ def set_fap(Far, f, tmp_path = None, verbose = False):
 	containing the usual inspiral tables.
 
 	Far = LocalRankingData class instance
-	f = filename of the databse (e.g.something.sqlite) 
+	f = filename of the databse (e.g.something.sqlite)
 	tmp_path = the local disk path to copy the database to in
-		order to avoid sqlite commands over nfs 
+		order to avoid sqlite commands over nfs
 	verbose = be verbose
 	"""
 	# FIXME this code should be moved into a method of the LocalRankingData class once other cleaning is done
