@@ -25,11 +25,22 @@
 #
 
 
+from collections import deque
 import numpy
 import os
+import resource
 from scipy import random
+try:
+	import sqlite3
+except ImportError:
+        # pre 2.5.x
+	from pysqlite2 import dbapi2 as sqlite3
 import StringIO
 import subprocess
+import sys
+import threading
+import time
+import httplib
 
 # The following snippet is taken from http://gstreamer.freedesktop.org/wiki/FAQ#Mypygstprogramismysteriouslycoredumping.2Chowtofixthis.3F
 import pygtk
@@ -41,45 +52,36 @@ pygst.require('0.10')
 import gst
 
 try:
-	import sqlite3
+	from ligo import gracedb
 except ImportError:
-        # pre 2.5.x
-	from pysqlite2 import dbapi2 as sqlite3
-import sys
-import threading
-import time
-from collections import deque
-import resource
-
-try:
-	from ligo.gracedb import cli as gracedb
-except ImportError:
-	print >>sys.stderr, "warning: gracedb import failed, gracedb uploads disabled"
+	print >>sys.stderr, "warning: gracedb import failed, program will crash if gracedb uploads are attempted"
 
 from glue import iterutils
 from glue import segments
 from glue.ligolw import ligolw
+from glue.ligolw import dbtables
 from glue.ligolw import ilwd
 from glue.ligolw import lsctables
-from glue.ligolw import dbtables
-from glue.ligolw import utils
+from glue.ligolw import array as ligolw_array
+from glue.ligolw import param as ligolw_param
+from glue.ligolw import utils as ligolw_utils
 from glue.ligolw.utils import ligolw_sqlite
 from glue.ligolw.utils import ligolw_add
 from glue.ligolw.utils import process as ligolw_process
 from glue.ligolw.utils import search_summary as ligolw_search_summary
+from pylal import datatypes as laltypes
+from pylal import ligolw_tisi
+from pylal import rate
 from pylal.datatypes import LIGOTimeGPS
 from pylal.datatypes import REAL8FrequencySeries
 from pylal.xlal.datatypes.snglinspiraltable import from_buffer as sngl_inspirals_from_buffer
-from pylal import ligolw_tisi
-from pylal import rate
+
 from gstlal import bottle
 from gstlal import reference_psd
 from gstlal import streamthinca
 from gstlal import svd_bank
 from gstlal import cbc_template_iir
 from gstlal import far
-from pylal import llwapp
-from pylal import datatypes as laltypes
 
 lsctables.LIGOTimeGPS = LIGOTimeGPS
 
@@ -320,19 +322,11 @@ def add_cbc_metadata(xmldoc, process, seg_in):
 
 class XMLContentHandler(ligolw.LIGOLWContentHandler):
 	pass
-
-rate.array.use_in(XMLContentHandler)
-rate.param.use_in(XMLContentHandler)
+ligolw_array.use_in(XMLContentHandler)
+ligolw_param.use_in(XMLContentHandler)
 lsctables.use_in(XMLContentHandler)
 
 
-class DBContentHandler(ligolw.LIGOLWContentHandler):
-	pass
-rate.array.use_in(DBContentHandler)
-rate.param.use_in(DBContentHandler)
-dbtables.use_in(DBContentHandler)
-
-
 #
 # =============================================================================
 #
@@ -393,7 +387,7 @@ def gen_likelihood_control_doc(far, instruments, name = u"gstlal_inspiral_likeli
 
 	node.appendChild(far.to_xml(process, name))
 
-	llwapp.set_process_end_time(process)
+	ligolw_process.set_process_end_time(process)
 	return xmldoc
 
 
@@ -555,7 +549,7 @@ class CoincsDocument(object):
 		else:
 			self.sngl_inspiral_table.sort(lambda a, b: cmp(a.end_time, b.end_time) or cmp(a.end_time_ns, b.end_time_ns) or cmp(a.ifo, b.ifo))
 			self.search_summary.nevents = len(self.sngl_inspiral_table)
-			utils.write_filename(self.xmldoc, self.filename, gz = (self.filename or "stdout").endswith(".gz"), verbose = verbose, trap_signals = None)
+			ligolw_utils.write_filename(self.xmldoc, self.filename, gz = (self.filename or "stdout").endswith(".gz"), verbose = verbose, trap_signals = None)
 
 
 class Data(object):
@@ -629,8 +623,7 @@ class Data(object):
 		self.ram_history = deque(maxlen = 1000)
 
 	def appsink_new_buffer(self, elem):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			# retrieve triggers from appsink element
 			buf = elem.emit("pull-buffer")
 			events = sngl_inspirals_from_buffer(buf)
@@ -664,7 +657,14 @@ class Data(object):
 			# update likelihood snapshot if needed
 			if self.likelihood_snapshot_interval is not None and (self.likelihood_snapshot_timestamp is None or (self.likelihood_snapshot_interval is not None and buf_timestamp - self.likelihood_snapshot_timestamp >= self.likelihood_snapshot_interval)):
 				self.likelihood_snapshot_timestamp = buf_timestamp
-				# Post a checkpoint message
+				# post a checkpoint message.  FIXME:  make
+				# sure this triggers
+				# self.snapshot_output_file() to be
+				# invoked.  lloidparts takes care of that
+				# for now, but spreading the program logic
+				# around like that isn't a good idea, this
+				# code should be responsible for it
+				# somehow, no?
 				self.pipeline.get_bus().post(message_new_checkpoint(self.pipeline, timestamp = buf_timestamp.ns()))
 				if self.assign_likelihoods:
 					assert self.marginalized_likelihood_file is not None
@@ -674,7 +674,7 @@ class Data(object):
 					self.stream_thinca.set_likelihood_data(self.far.distribution_stats.smoothed_distributions, self.far.distribution_stats.likelihood_params_func)
 
 					# Read in the the background likelihood distributions that should have been updated asynchronously
-					self.ranking_data, procid = far.RankingData.from_xml(utils.load_filename(self.marginalized_likelihood_file, verbose = self.verbose, contenthandler = XMLContentHandler))
+					self.ranking_data, procid = far.RankingData.from_xml(ligolw_utils.load_filename(self.marginalized_likelihood_file, verbose = self.verbose, contenthandler = XMLContentHandler))
 					self.ranking_data.compute_joint_cdfs()
 
 					# set up the scale factor for the trials table to normalize the rate
@@ -686,7 +686,7 @@ class Data(object):
 							print >>sys.stderr, "could not set scale factor, probably because we do not have live time info yet.  seg is: %s" % str(self.ranking_data.livetime_seg)
 
 					# write the new distribution stats to disk
-					utils.write_filename(gen_likelihood_control_doc(self.far, self.instruments), self.likelihood_file, gz = (self.likelihood_file or "stdout").endswith(".gz"), verbose = False, trap_signals = None)
+					ligolw_utils.write_filename(gen_likelihood_control_doc(self.far, self.instruments), self.likelihood_file, gz = (self.likelihood_file or "stdout").endswith(".gz"), verbose = False, trap_signals = None)
 				else:
 					self.ranking_data = None
 
@@ -705,24 +705,18 @@ class Data(object):
 			if self.gracedb_far_threshold is not None:
 				self.__do_gracedb_alerts()
 				self.__update_eye_candy()
-		finally:
-			self.lock.release()
 
-	def __write_likelihood_file(self):
-		# write the new distribution stats to disk
+	def __web_get_likelihood_file(self):
+		# generate a coinc parameter distribution file
 		output = StringIO.StringIO()
-		utils.write_fileobj(gen_likelihood_control_doc(self.far, self.instruments), output, trap_signals = None)
+		ligolw_utils.write_fileobj(gen_likelihood_control_doc(self.far, self.instruments), output, trap_signals = None)
 		outstr = output.getvalue()
 		output.close()
 		return outstr
 
 	def web_get_likelihood_file(self):
-		self.lock.acquire()
-		try:
-			outstr = self.__write_likelihood_file()
-		finally:
-			self.lock.release()
-		return outstr
+		with self.lock:
+			return self.__web_get_likelihood_file()
 
 	def __flush(self):
 		# run StreamThinca's .flush().  returns the last remaining
@@ -740,27 +734,15 @@ class Data(object):
 			self.__do_gracedb_alerts()
 
 	def flush(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			self.__flush()
-		finally:
-			self.lock.release()
 
 	def __do_gracedb_alerts(self):
-		try:
-			gracedb
-		except NameError:
-			# gracedb import failed, disable event uploads
-			return
 		if self.stream_thinca.last_coincs:
 			gracedb_client = gracedb.Client()
 			gracedb_ids = []
 			psdmessage = None
 			coinc_inspiral_index = self.stream_thinca.last_coincs.coinc_inspiral_index
-			# FIXME:  this should maybe not be retrieved this
-			# way.  and the .column_index() method is probably
-			# useless
-			likelihood_dict = self.stream_thinca.last_coincs.column_index(lsctables.CoincTable.tableName, "likelihood")
 
 			# FIXME:  this is hacked to only send at most the
 			# one best coinc in this set.  May not make sense
@@ -769,7 +751,7 @@ class Data(object):
 			# we should be doing coincidences every 10s which
 			# is a reasonable time to cluster over.  the slice
 			# can be edited (or removed) to change this.
-			for likelihood, coinc_event_id in sorted((likelihood, coinc_event_id) for (coinc_event_id, likelihood) in likelihood_dict.items())[-1:]:
+			for likelihood, coinc_event_id in sorted((coinc_event.likelihood, coinc_event.coinc_event_id) for coinc_event in self.stream_thinca.last_coincs.coinc_event_index.values())[-1:]:
 				#
 				# quit if the false alarm rate is not low
 				# enough, or is nan
@@ -823,18 +805,17 @@ class Data(object):
 				if self.verbose:
 					print >>sys.stderr, "sending %s to gracedb ..." % filename
 				message = StringIO.StringIO()
-				utils.write_fileobj(self.stream_thinca.last_coincs[coinc_event_id], message, gz = False, trap_signals = None)
+				ligolw_utils.write_fileobj(self.stream_thinca.last_coincs[coinc_event_id], message, gz = False, trap_signals = None)
 				# FIXME: make this optional from command line?
 				if True:
-					resp = gracedb_client.create(self.gracedb_group, self.gracedb_type, filename, message.getvalue())
-					if "error" in resp:
-						print >>sys.stderr, "gracedb upload of %s failed: %s" % (filename, resp["error"])
+					resp = gracedb_client.createEvent(self.gracedb_group, self.gracedb_type, filename, message.getvalue())
+					resp_json = resp.json()
+					if resp.status != httplib.CREATED:
+						print >>sys.stderr, "gracedb upload of %s failed" % filename
 					else:
 						if self.verbose:
-							if "warning" in resp:
-								print >>sys.stderr, "gracedb issued warning: %s" % resp["warning"]
-							print >>sys.stderr, "event assigned grace ID %s" % resp["graceid"]
-						gracedb_ids.append(resp["graceid"])
+							print >>sys.stderr, "event assigned grace ID %s" % resp_json["graceid"]
+						gracedb_ids.append(resp_json["graceid"])
 				else:
 					proc = subprocess.Popen(("/bin/cp", "/dev/stdin", filename), stdin = subprocess.PIPE)
 					proc.stdin.write(message.getvalue())
@@ -849,16 +830,14 @@ class Data(object):
 			if psdmessage is not None:
 				filename = "psd.xml.gz"
 				for gracedb_id in gracedb_ids:
-					resp = gracedb_client.upload(gracedb_id, filename, psdmessage.getvalue(), comment = "strain spectral densities", alert = True)
-					if "error" in resp:
-						print >>sys.stderr, "gracedb upload of %s for ID %s failed: %s" % (filename, gracedb_id, resp["error"])
+					resp = gracedb_client.writeLog(gracedb_id, "strain spectral densities", filename = filename, filecontents = psdmessage.getvalue(), tagname = "psd")
+					resp_json = resp.json()
+					if resp.status != httplib.CREATED:
+						print >>sys.stderr, "gracedb upload of %s for ID %s failed" % (filename, gracedb_id)
 
 	def do_gracedb_alerts(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			self.__do_gracedb_alerts()
-		finally:
-			self.lock.release()
 
 	def __update_eye_candy(self):
 		if self.stream_thinca.last_coincs:
@@ -866,7 +845,9 @@ class Data(object):
 			latency_val = None
 			snr_val = (0,0)
 			coinc_inspiral_index = self.stream_thinca.last_coincs.coinc_inspiral_index
-			for coinc_event_id, latency in self.stream_thinca.last_coincs.column_index(lsctables.CoincInspiralTable.tableName, "minimum_duration").items():
+			for coinc_event_id, coinc_inspiral in coinc_inspiral_index.items():
+				# FIXME:  update when a proper column is available
+				latency = coinc_inspiral.minimum_duration
 				self.latency_histogram[latency,] += 1
 				if latency_val is None:
 					t = float(coinc_inspiral_index[coinc_event_id].get_end())
@@ -881,105 +862,71 @@ class Data(object):
 				self.snr_history.append(snr_val)
 
 	def update_eye_candy(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			self.__update_eye_candy()
-		finally:
-			self.lock.release()
-
 
 	def web_get_latency_histogram(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			for latency, number in zip(self.latency_histogram.centres()[0][1:-1], self.latency_histogram.array[1:-1]):
 				yield "%e %e\n" % (latency, number)
-		finally:
-			self.lock.release()
-
 
 	def web_get_latency_history(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			# first one in the list is sacrificed for a time stamp
 			for time, latency in self.latency_history:
 				yield "%f %e\n" % (time, latency)
-		finally:
-			self.lock.release()
-
 
 	def web_get_snr_history(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			# first one in the list is sacrificed for a time stamp
 			for time, snr in self.snr_history:
 				yield "%f %e\n" % (time, snr)
-		finally:
-			self.lock.release()
-
 
 	def web_get_ram_history(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			# first one in the list is sacrificed for a time stamp
 			for time, ram in self.ram_history:
 				yield "%f %e\n" % (time, ram)
-		finally:
-			self.lock.release()
-
 
 	def web_get_gracedb_far_threshold(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			if self.gracedb_far_threshold is not None:
 				yield "rate=%.17g\n" % self.gracedb_far_threshold
 			else:
 				yield "rate=\n"
-		finally:
-			self.lock.release()
-
 
 	def web_set_gracedb_far_threshold(self):
-		self.lock.acquire()
 		try:
-			rate = bottle.request.forms["rate"]
-			if rate:
-				self.gracedb_far_threshold = float(rate)
-				yield "OK: rate=%.17g\n" % self.gracedb_far_threshold
-			else:
-				self.gracedb_far_threshold = None
-				yield "OK: rate=\n"
+			with self.lock:
+				rate = bottle.request.forms["rate"]
+				if rate:
+					self.gracedb_far_threshold = float(rate)
+					yield "OK: rate=%.17g\n" % self.gracedb_far_threshold
+				else:
+					self.gracedb_far_threshold = None
+					yield "OK: rate=\n"
 		except:
 			yield "error\n"
-		finally:
-			self.lock.release()
-
 
 	def web_get_sngls_snr_threshold(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			if self.stream_thinca.sngls_snr_threshold is not None:
 				yield "snr=%.17g\n" % self.stream_thinca.sngls_snr_threshold
 			else:
 				yield "snr=\n"
-		finally:
-			self.lock.release()
-
 
 	def web_set_sngls_snr_threshold(self):
-		self.lock.acquire()
 		try:
-			snr_threshold = bottle.request.forms["snr"]
-			if snr_threshold:
-				self.stream_thinca.sngls_snr_threshold = float(rate)
-				yield "OK: snr=%.17g\n" % self.stream_thinca.sngls_snr_threshold
-			else:
-				self.stream_thinca.sngls_snr_threshold = None
-				yield "OK: snr=\n"
+			with self.lock:
+				snr_threshold = bottle.request.forms["snr"]
+				if snr_threshold:
+					self.stream_thinca.sngls_snr_threshold = float(rate)
+					yield "OK: snr=%.17g\n" % self.stream_thinca.sngls_snr_threshold
+				else:
+					self.stream_thinca.sngls_snr_threshold = None
+					yield "OK: snr=\n"
 		except:
 			yield "error\n"
-		finally:
-			self.lock.release()
-
 
 	def __write_output_file(self, filename = None, likelihood_file = None, verbose = False):
 		self.__flush()
@@ -997,23 +944,15 @@ class Data(object):
 				fname = os.path.join(fname[0], '%s_SNR_CHI.xml.gz' % fname[1].split('.')[0])
 		else:
 			fname = likelihood_file
-		utils.write_filename(gen_likelihood_control_doc(self.far, self.instruments), fname, gz = (fname or "stdout").endswith(".gz"), verbose = verbose, trap_signals = None)
-
+		ligolw_utils.write_filename(gen_likelihood_control_doc(self.far, self.instruments), fname, gz = (fname or "stdout").endswith(".gz"), verbose = verbose, trap_signals = None)
 
 	def write_output_file(self, filename = None, likelihood_file = None, verbose = False):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			self.__write_output_file(filename = filename, likelihood_file = likelihood_file, verbose = verbose)
 			# can't be used anymore
 			del self.coincs_document
-		finally:
-			self.lock.release()
-
 
 	def snapshot_output_file(self, description, extension, verbose = False):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			self.__write_output_file(filename = self.coincs_document.T050017_filename(description, extension), verbose = verbose)
 			self.coincs_document = self.coincs_document.get_another()
-		finally:
-			self.lock.release()
