@@ -240,66 +240,19 @@ class Handler(simplehandler.Handler):
 		xmldoc.appendChild(ligolw.LIGO_LW())
 		ligolwsegments = ligolw_segments.LigolwSegments(xmldoc)
 		process = ligolw_process.register_to_xmldoc(xmldoc, "gstlal_inspiral", {})
-		ligolwsegments.insert_from_segmentlistdict(self.segments, name = "datasegments", version = None, insertion_time = None, comment = "LLOID snapshot")
+		ligolwsegments.insert_from_segmentlistdict(self.segments, name = "datasegments", version = None, comment = "LLOID snapshot")
 		ligolwsegments.finalize(process)
 		return xmldoc
 
 	def web_get_segments_xml(self):
-		self.lock.acquire()
-		try:
+		with self.lock:
 			output = StringIO.StringIO()
 			utils.write_fileobj(self.gen_segments_doc(), output, trap_signals = None)
 			outstr = output.getvalue()
 			output.close()
-		finally:
-			self.lock.release()
 		return outstr
 
 
-#
-# data source
-#
-
-
-def mkLLOIDbasicsrc(pipeline, seekevent, instrument, detector, data_source = "frames", injection_filename = None, frame_segments = None, state_vector_on_off_dict = {"H1" : (0x7, 0x160), "L1" : (0x7, 0x160), "V1" : (0x67, 0x100)}, verbose = False):
-
-	#
-	# FIXME this is deprecated, call datasource.mkbasicsrc directly.  THIS WILL BE DELETED!
-	#
-
-	warnings.warn("mkLLOIDbasicsrc() is obsolete.  Please modify your code to call datasource.mkbasicsrc as this function will be deleted in a future release", DeprecationWarning)
-
-	class gwinfo(object):
-		def __init__(self, seekevent, instrument, detector, data_source, injection_filename, frame_segments, state_vector_on_off_dict):
-			self.channel_dict = {instrument: detector.channel}
-			self.frame_segments = frame_segments or dict([(instrument, None) for instrument in self.channel_dict])
-			self.seekevent = seekevent
-			self.state_vector_on_off_bits = state_vector_on_off_dict
-			self.data_source = data_source
-			self.frame_cache = detector.frame_cache
-			self.block_size = detector.block_size
-			self.injection_filename = injection_filename
-			self.gate_start_callback = None
-			self.gate_stop_callback = None
-			self.dq_channel_type = "LLD"
-			self.shm_part_dict = {"H1": "LHO_Data", "H2": "LHO_Data", "L1": "LLO_Data", "V1": "VIRGO_Data"}
-			self.dq_channel_dict = { "H1": "LLD-DQ_VECTOR", "H2": "LLD-DQ_VECTOR","L1": "LLD-DQ_VECTOR", "V1": "LLD-DQ_VECTOR" }
-
-	return datasource.mkbasicsrc(pipeline, gwinfo(seekevent, instrument, detector, data_source, injection_filename, frame_segments, state_vector_on_off_dict), instrument, verbose)
-
-
-def mkLLOIDsrc(pipeline, src, rates, instrument, psd = None, psd_fft_length = 8, ht_gate_threshold = None, veto_segments = None, seekevent = None, nxydump_segment = None, track_psd = False, zero_pad = 0):
-	"""Build pipeline stage to whiten and downsample h(t)."""
-
-	#
-	# FIXME this function is deprecated, should be deleted
-	#
-
-	warnings.warn("mkLLOIDsrc() is deprecated.  Call multirate_datasource.mkwhitened_multirate_src() instead", DeprecationWarning)
-
-	return multirate_datasource.mkwhitened_multirate_src(pipeline, src, rates, instrument, psd = psd, psd_fft_length = psd_fft_length, ht_gate_threshold = ht_gate_threshold, veto_segments = veto_segments, seekevent = seekevent, nxydump_segment = nxydump_segment, track_psd = track_psd, zero_pad = zero_pad, width = 32)
-
-
 #
 # one instrument, one template bank
 #
@@ -445,7 +398,7 @@ def mkLLOIDhoftToSnrSlices(pipeline, hoftdict, bank, control_snksrc, block_durat
 			# firbank element, and the value here is only
 			# approximate and not tied to the fir bank
 			# parameters so might not work if those change
-			pipeparts.mkqueue(pipeline, pipeparts.mkdrop(pipeline, hoftdict[bank_fragment.rate], int(round((bank.filter_length - bank_fragment.end) * bank_fragment.rate))), max_size_bytes = 0, max_size_buffers = 0, max_size_time = (1 * fir_stride + int(math.ceil(bank.filter_length))) * gst.SECOND),			
+			pipeparts.mkqueue(pipeline, pipeparts.mkdrop(pipeline, hoftdict[bank_fragment.rate], int(round((bank.filter_length - bank_fragment.end) * bank_fragment.rate))), max_size_bytes = 0, max_size_buffers = 0, max_size_time = (1 * fir_stride + int(math.ceil(bank.filter_length))) * gst.SECOND),
 			bank,
 			bank_fragment,
 			control_snksrc,
@@ -614,33 +567,6 @@ def mkLLOIDSnrSlicesToTimeSliceChisq(pipeline, branch_heads, bank, block_duratio
 	return pipeparts.mkqueue(pipeline, chisq, max_size_bytes = 0, max_size_buffers = 0, max_size_time = block_duration)
 
 
-def mkLLOIDSnrToAutoChisq(pipeline, snr, bank):
-	"""Build pipeline fragment that computes the AutoChisq from single detector SNR."""
-	#
-	# parameters
-	#
-
-	autocorrelation_length = bank.autocorrelation_bank.shape[1]
-	autocorrelation_latency = -(autocorrelation_length - 1) / 2
-
-	# FIXME something like this could be tried.
-	#mask_matrix = numpy.ones(bank.autocorrelation_bank.shape, numpy.int)
-	#stix = autocorrelation_latency - 10
-	#mask_matrix[:,stix:] = 0
-
-	#
-	# \chi^{2}
-	#
-
-	chisq = pipeparts.mkautochisq(pipeline, snr, autocorrelation_matrix = bank.autocorrelation_bank, mask_matrix = None, latency = autocorrelation_latency, snr_thresh = bank.snr_threshold)
-	chisq = pipeparts.mkchecktimestamps(pipeline, chisq, "timestamps_%s_after_chisq" % bank.logname)
-
-	#chisq = pipeparts.mktee(pipeline, chisq)
-	#pipeparts.mknxydumpsink(pipeline, pipeparts.mkqueue(pipeline, chisq), "chisq_%s.dump" % logname, segment = nxydump_segment)
-
-	return chisq
-
-
 def mkLLOIDSnrChisqToTriggers(pipeline, snr, chisq, bank, verbose = False, nxydump_segment = None, logname = ""):
 	"""Build pipeline fragment that converts single detector SNR and Chisq into triggers."""
 	#
@@ -701,9 +627,9 @@ def mkLLOIDmulti(pipeline, detectors, banks, psd, psd_fft_length = 8, ht_gate_th
 		rates = set(rate for bank in banks[instrument] for rate in bank.get_rates()) # FIXME what happens if the rates are not the same?
 		src = datasource.mkbasicsrc(pipeline, detectors, instrument, verbose)
 		if veto_segments is not None:
-			hoftdicts[instrument] = mkLLOIDsrc(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, seekevent = detectors.seekevent, ht_gate_threshold = ht_gate_threshold, veto_segments = veto_segments[instrument], nxydump_segment = nxydump_segment, track_psd = track_psd)
+			hoftdicts[instrument] = multirate_datasource.mkwhitened_multirate_src(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, ht_gate_threshold = ht_gate_threshold, veto_segments = veto_segments[instrument], seekevent = detectors.seekevent, nxydump_segment = nxydump_segment, track_psd = track_psd, zero_pad = 0, width = 32)
 		else:
-			hoftdicts[instrument] = mkLLOIDsrc(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, seekevent = detectors.seekevent, ht_gate_threshold = ht_gate_threshold, nxydump_segment = nxydump_segment, track_psd = track_psd)
+			hoftdicts[instrument] = multirate_datasource.mkwhitened_multirate_src(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, ht_gate_threshold = ht_gate_threshold, veto_segments = None, seekevent = detectors.seekevent, nxydump_segment = nxydump_segment, track_psd = track_psd, zero_pad = 0, width = 32)
 
 	#
 	# build gate control branches
@@ -751,40 +677,24 @@ def mkLLOIDmulti(pipeline, detectors, banks, psd, psd_fft_length = 8, ht_gate_th
 			snrslices = snrslices
 		)
 		snr = pipeparts.mkchecktimestamps(pipeline, snr, "timestamps_%s_snr" % suffix)
-		# FIXME you get a different trigger generator depending on the chisq calculation :/
 		if chisq_type == 'autochisq':
 			# FIXME don't hardcode
-			# peak finding window (n) in samples is four seconds at max rate, ie max(rates)
-			head = pipeparts.mkitac(pipeline, snr, 1 * max(rates), bank.template_bank_filename, autocorrelation_matrix = bank.autocorrelation_bank, snr_thresh = bank.snr_threshold, sigmasq = bank.sigmasq)
+			# peak finding window (n) in samples is 1 second at max rate, ie max(rates)
+			head = pipeparts.mkitac(pipeline, snr, 1 * max(rates), bank.template_bank_filename, autocorrelation_matrix = bank.autocorrelation_bank, mask_matrix = bank.autocorrelation_mask, snr_thresh = bank.snr_threshold, sigmasq = bank.sigmasq)
 			if verbose:
 				head = pipeparts.mkprogressreport(pipeline, head, "progress_xml_%s" % suffix)
 			triggersrcs.add(head)
-			# old way
-			# snr = pipeparts.mktee(pipeline, snr)
-			# chisq = mkLLOIDSnrToAutoChisq(pipeline, pipeparts.mkqueue(pipeline, snr, max_size_buffers = 0, max_size_bytes = 0, max_size_time = 1 * block_duration), bank)
 		else:
-			chisq = mkLLOIDSnrSlicesToTimeSliceChisq(pipeline, snrslices, bank, block_duration)
-			triggersrcs.add(mkLLOIDSnrChisqToTriggers(
-				pipeline,
-				pipeparts.mkqueue(pipeline, snr, max_size_bytes = 0, max_size_buffers = 0, max_size_time = 1 * block_duration),
-				chisq,
-				bank,
-				verbose = verbose,
-				nxydump_segment = nxydump_segment,
-				logname = suffix
-			))
+			raise NotImplementedError("Currently only 'autochisq' is supported")
 		# FIXME:  find a way to use less memory without this hack
 		del bank.autocorrelation_bank
 		#pipeparts.mknxydumpsink(pipeline, pipeparts.mktogglecomplex(pipeline, pipeparts.mkqueue(pipeline, snr)), "snr_%s.dump" % suffix, segment = nxydump_segment)
 		#pipeparts.mkogmvideosink(pipeline, pipeparts.mkcapsfilter(pipeline, pipeparts.mkchannelgram(pipeline, pipeparts.mkqueue(pipeline, snr), plot_width = .125), "video/x-raw-rgb, width=640, height=480, framerate=64/1"), "snr_channelgram_%s.ogv" % suffix, audiosrc = pipeparts.mkaudioamplify(pipeline, pipeparts.mkqueue(pipeline, hoftdict[max(bank.get_rates())], max_size_time = 2 * int(math.ceil(bank.filter_length)) * gst.SECOND), 0.125), verbose = True)
 
 	#
-	# if there is more than one trigger source, synchronize the streams
-	# with a multiqueue then use an n-to-1 adapter to combine into a
-	# single stream
+	# done
 	#
 
-
 	assert len(triggersrcs) > 0
 	return triggersrcs
 
@@ -827,9 +737,9 @@ def mkSPIIRmulti(pipeline, detectors, banks, psd, psd_fft_length = 8, ht_gate_th
 		rates = set(rate for bank in banks[instrument] for rate in bank.get_rates()) # FIXME what happens if the rates are not the same?
 		src = datasource.mkbasicsrc(pipeline, detectors, instrument, verbose)
 		if veto_segments is not None:
-			hoftdicts[instrument] = mkLLOIDsrc(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, seekevent = detectors.seekevent, ht_gate_threshold = ht_gate_threshold, veto_segments = veto_segments[instrument], nxydump_segment = nxydump_segment, track_psd = track_psd)
+			hoftdicts[instrument] = multirate_datasource.mkwhitened_multirate_src(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, ht_gate_threshold = ht_gate_threshold, veto_segments = veto_segments[instrument], seekevent = detectors.seekevent, nxydump_segment = nxydump_segment, track_psd = track_psd, zero_pad = 0, width = 32)
 		else:
-			hoftdicts[instrument] = mkLLOIDsrc(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, seekevent = detectors.seekevent, ht_gate_threshold = ht_gate_threshold, nxydump_segment = nxydump_segment, track_psd = track_psd)
+			hoftdicts[instrument] = multirate_datasource.mkwhitened_multirate_src(pipeline, src, rates, instrument, psd = psd[instrument], psd_fft_length = psd_fft_length, ht_gate_threshold = ht_gate_threshold, veto_segments = None, seekevent = detectors.seekevent, nxydump_segment = nxydump_segment, track_psd = track_psd, zero_pad = 0, width = 32)
 
 	#
 	# construct trigger generators
@@ -856,58 +766,20 @@ def mkSPIIRmulti(pipeline, detectors, banks, psd, psd_fft_length = 8, ht_gate_th
 		if chisq_type == 'autochisq':
 			# FIXME don't hardcode
 			# peak finding window (n) in samples is one second at max rate, ie max(rates)
-			head = pipeparts.mkitac(pipeline, snr, max(rates), bank.template_bank_filename, autocorrelation_matrix = bank.autocorrelation_bank, snr_thresh = bank.snr_threshold, sigmasq = bank.sigmasq)
+			head = pipeparts.mkitac(pipeline, snr, max(rates), bank.template_bank_filename, autocorrelation_matrix = bank.autocorrelation_bank, mask_matrix = bank.autocorrelation_mask, snr_thresh = bank.snr_threshold, sigmasq = bank.sigmasq)
 			if verbose:
 				head = pipeparts.mkprogressreport(pipeline, head, "progress_xml_%s" % suffix)
 			triggersrcs.add(head)
-			# old way
-			# chisq = mkLLOIDSnrToAutoChisq(pipeline, pipeparts.mkqueue(pipeline, snr, max_size_buffers = 0, max_size_bytes = 0, max_size_time = 1 * block_duration), bank)
 		# FIXME:  find a way to use less memory without this hack
 		del bank.autocorrelation_bank
 		#pipeparts.mknxydumpsink(pipeline, pipeparts.mktogglecomplex(pipeline, pipeparts.mkqueue(pipeline, snr)), "snr_%s.dump" % suffix, segment = nxydump_segment)
 		#pipeparts.mkogmvideosink(pipeline, pipeparts.mkcapsfilter(pipeline, pipeparts.mkchannelgram(pipeline, pipeparts.mkqueue(pipeline, snr), plot_width = .125), "video/x-raw-rgb, width=640, height=480, framerate=64/1"), "snr_channelgram_%s.ogv" % suffix, audiosrc = pipeparts.mkaudioamplify(pipeline, pipeparts.mkqueue(pipeline, hoftdict[max(bank.get_rates())], max_size_time = 2 * int(math.ceil(bank.filter_length)) * gst.SECOND), 0.125), verbose = True)
-## 		triggersrc.add(mkLLOIDSnrChisqToTriggers(
-## 			pipeline,
-## 			pipeparts.mkqueue(pipeline, snr, max_size_bytes = 0, max_size_buffers = 0, max_size_time = 1 * block_duration),
-## 			chisq,
-## 			bank,
-## 			verbose = verbose,
-## 			nxydump_segment = nxydump_segment,
-## 			logname = suffix
-## 		))
-
-	#
-	# if there is more than one trigger source, synchronize the streams
-	# with a multiqueue then use an n-to-1 adapter to combine into a
-	# single stream
-	#
-
-	assert len(triggersrcs) > 0
-## 	if len(triggersrc) > 1:
-## 		# send all streams through a multiqueue
-## 		queue = gst.element_factory_make("multiqueue")
-## 		pipeline.add(queue)
-## 		for head in triggersrc:
-## 			head.link(queue)
-## 		triggersrc = queue
-## 		# FIXME:  it has been reported that the input selector
-## 		# breaks seeks.  confirm and fix if needed
-## 		# FIXME:  input-selector in 0.10.32 no longer has the
-## 		# "select-all" feature.  need to get this re-instated
-## 		#nto1 = gst.element_factory_make("input-selector")
-## 		#nto1.set_property("select-all", True)
-## 		#pipeline.add(nto1)
-## 		#for pad in queue.src_pads():
-## 		#	pad.link(nto1)
-## 		#triggersrc = nto1
-## 	else:
-## 		# len(triggersrc) == 1
-## 		triggersrc, = triggersrc
 
 	#
 	# done
 	#
 
+	assert len(triggersrcs) > 0
 	return triggersrcs
 
 
